<?php
	
	if (!defined('__IN_SYMPHONY__')) die('<h2>Symphony Error</h2><p>You cannot directly access this file</p>');
	
	require_once(TOOLKIT . '/fields/field.upload.php');
	
	class FieldImageUploadPreview extends FieldUpload {
		public function __construct(&$parent){
			parent::__construct($parent);
			$this->_name = 'Image Upload & Preview';
		}
		
		private function getHashedFilename($filename) {
			preg_match('/\.[^\.]+$/', $filename, $meta);
			return md5(time() . $filename) . $meta[0];
		}
		
		public function checkPostFieldData($data, &$message, $entry_id = null) {
			if (is_array($data) and isset($data['name'])) $data['name'] = $this->getHashedFilename($data['name']);
			return parent::checkPostFieldData($data, $message, $entry_id);
		}
		
		public function processRawFieldData($data, &$status, $simulate = false, $entry_id = null) {
			if (is_array($data) and isset($data['name'])) $data['name'] = $this->getHashedFilename($data['name']);
			return parent::processRawFieldData($data, $status, $simulate, $entry_id);
		}
		
		function appendFormattedElement(&$wrapper, $data){
			$item = new XMLElement($this->get('element_name'));

			$item->appendChild(new XMLElement('filename', General::sanitize(basename($data['file']))));
			
			$item->setAttributeArray(array(
				'size' => General::formatFilesize(filesize(WORKSPACE . $data['file'])),
			 	'path' => str_replace(WORKSPACE, NULL, dirname(WORKSPACE . $data['file'])),
				'type' => $data['mimetype'],
			));
						
			$m = unserialize($data['meta']);
			
			if(is_array($m) && !empty($m)){
				$item->appendChild(new XMLElement('meta', NULL, $m));
			}
			
			$preview_info = $this->Database->fetchRow(0, "SELECT width, height, crop FROM sym_fields_imageuploadpreview WHERE field_id='{$this->_fields['id']}'");
			
			$preview = new XMLElement('preview');
			$preview->setAttributeArray(array(
				'width' => $preview_info['width'],
				'height' => $preview_info['height'],
				'crop' => $preview_info['crop']
			));
			$item->appendChild($preview);
			
			$wrapper->appendChild($item);
		}
		
		function commit(){

			if(!parent::commit()) return false;
			
			$id = $this->get('id');

			if($id === false) return false;
			
			$fields = array();
			
			$fields['field_id'] = $id;
			
			$fields['destination'] = $this->get('destination');
			$fields['validator'] = ($fields['validator'] == 'custom' ? NULL : $this->get('validator'));
			
			$fields['width'] = $this->get('width');
			$fields['height'] = $this->get('height');
			$fields['crop'] = $this->get('crop');
			
			$this->_engine->Database->query("DELETE FROM `tbl_fields_".$this->handle()."` WHERE `field_id` = '$id' LIMIT 1");
			return $this->_engine->Database->insert($fields, 'tbl_fields_' . $this->handle());					
			
		}
		
		public function displayPublishPanel(&$wrapper, $data=NULL, $flagWithError=NULL, $fieldnamePrefix=NULL, $fieldnamePostfix=NULL){

			if(!$flagWithError && !is_writable(DOCROOT . $this->get('destination') . '/')) 
				$flagWithError = 'Destination folder, <code>'.$this->get('destination').'</code>, is not writable. Please check permissions.';
			
			$label_text = $this->get('label');
			if ($data['file']) {
				$label_text .= " (" . $this->get("width") . "x" . $this->get("height") . " preview, <a href=\"" . URL . "/workspace" . $data['file'] . "\" style=\"float:none;\">view original</a>)";
			}
			
			$label = Widget::Label($label_text);
			$class = 'file';
			$label->setAttribute('class', $class);
			if($this->get('required') != 'yes') $label->appendChild(new XMLElement('i', 'Optional'));
			
			$span = new XMLElement('span');
			if ($data['file']) {
				$img = new XMLElement("img");
				$img->setAttribute("alt", "");
				$img->setAttribute("src", URL . '/image/2/' . $this->get("width") . '/' . $this->get("height") . '/' . $this->get("crop") . '' . $data['file']);
				$span->appendChild($img);
			}
			
			$span->appendChild(Widget::Input('fields'.$fieldnamePrefix.'['.$this->get('element_name').']'.$fieldnamePostfix, $data['file'], ($data['file'] ? 'hidden' : 'file')));
			
			$label->appendChild($span);
			
			if($flagWithError != NULL) $wrapper->appendChild(Widget::wrapFormElementWithError($label, $flagWithError));
			else $wrapper->appendChild($label);

		}
		
		public function checkFields(&$errors, $checkForDuplicates=true){
			
			if(!is_array($errors)) $errors = array();
			
			if($this->get('label') == '') $errors['label'] = 'This is a required field.';
			if($this->get('width') == '') $errors['width'] = 'This is a required field.';
			if($this->get('height') == '') $errors['height'] = 'This is a required field.';

			if($this->get('element_name') == '') $errors['element_name'] = 'This is a required field.';
			elseif(!preg_match('/^[A-z]([\w\d-_\.]+)?$/i', $this->get('element_name'))){
				$errors['element_name'] = 'Invalid element name. Must be valid QName.';
			}
			
			elseif($checkForDuplicates){
				$sql = "SELECT * FROM `tbl_fields` 
						WHERE `element_name` = '" . $this->get('element_name') . "'
						".($this->get('id') ? " AND `id` != '".$this->get('id')."' " : '')." 
						AND `parent_section` = '". $this->get('parent_section') ."' LIMIT 1";

				if($this->Database->fetchRow(0, $sql)){
					$errors['element_name'] = 'A field with that element name already exists. Please choose another.';
				}
			}

			return (is_array($errors) && !empty($errors) ? self::__ERROR__ : self::__OK__);
			
		}
		
		public function displaySettingsPanel(&$wrapper, $errors=NULL){
			
			parent::displaySettingsPanel($wrapper, $errors);

			$group = new XMLElement('div');
			$group->setAttribute('class', 'group');

			$label = Widget::Label('Width');
			$label->appendChild(Widget::Input('fields['.$this->get('sortorder').'][width]', $this->get('width')));
			if(isset($errors['width'])) $group->appendChild(Widget::wrapFormElementWithError($label, $errors['width']));
			else $group->appendChild($label);
			
			$label = Widget::Label('Height');
			$label->appendChild(Widget::Input('fields['.$this->get('sortorder').'][height]', $this->get('height')));
			if(isset($errors['height'])) $group->appendChild(Widget::wrapFormElementWithError($label, $errors['height']));
			else $group->appendChild($label);
						
			$wrapper->appendChild($group);
			
			$label = Widget::Label('Crop mode');
			$selected = $this->get('crop');
			$options = array(
				array('1', $selected == '1', 'Top left'),
				array('2', $selected == '2', 'Top centre'),
				array('3', $selected == '3', 'Top right'),
				array('4', $selected == '4', 'Middle left'),
				array('5', $selected == '5', 'Centre'),
				array('6', $selected == '6', 'Middle right'),
				array('7', $selected == '7', 'Bottom left'),
				array('8', $selected == '8', 'Bottom centre'),
				array('9', $selected == '9', 'Bottom right')
			);
			$label->appendChild(Widget::Select('fields['.$this->get('sortorder').'][crop]', $options));
			
			$wrapper->appendChild($label);
									
		}
		
	}
	
?>